﻿''' <summary>
''' Repräsentiert einen einzelnen Ultraschall-Transmitter.
''' </summary>
Public Class Transmitter
   Public Const PI2 = Math.PI / 2 ' = 90°
   Public PositionX As Single ' Position des Transmitters in mm vom Koordinantenursprung (Y ist immer 0).
   Public Const Width = 10 ' Breite des Transmitters in mm (nur zum Zeichnen).
   Public Const Height = 10 ' Höhe des Transmitters in mm (nur zum Zeichnen).
   Public Const Lambda = 8.6 ' Wellenlänge in mm
   Public Const c = 342 ' Schallgeschwindigkeit in m/s

   ''' <summary>
   ''' Initialisiert eine neue Instanz der Transmitter-Klasse
   ''' </summary>
   ''' <param name="x">Position des Mittelpunkt auf der X-Achse</param>
   Sub New(x As Single)
      PositionX = x
   End Sub

   ''' <summary>
   ''' Liefert Amplitude und Phase am angegeben Punkt.
   ''' </summary>
   ''' <param name="x">X-Koodrinate des Punkts, an dem Amplitude und Phase ermittelt werden sollen.</param>
   ''' <param name="y">Y-Koodrinate des Punkts, an dem Amplitude und Phase ermittelt werden sollen.</param>
   ''' <param name="PhaseShift">Pahsenverschiebung in Radiant.</param>
   ''' <returns>Amplitude und Phase am angegeben Punkt.</returns>
   Function getWaveVector(x As Double, y As Double, PhaseShift As Double) As WaveVector
      Dim Amplitude As Double = 1.0
      Dim Distance As Double = Math.Sqrt((PositionX - x) ^ 2 + y ^ 2)
      Dim Phase As Double = Distance / Lambda * 2 * Math.PI + PhaseShift
      Return New WaveVector(Amplitude * WinkelAbhängigkeit(x, y), Phase)
   End Function

   ''' <summary>
   ''' Berechnet die relative Amplitude in Abhängigkeit des Winkles des Messpunkts zum Transmitter.
   ''' </summary>
   ''' <param name="x">X-Koordinate des Messpunkts, globale Koordinaten.</param>
   ''' <param name="y">X-Koordinate des Messpunkts, globale Koordinaten.</param>
   ''' <returns>Die Winkelabhängige relative Amplitude [0..1]</returns>
   Function WinkelAbhängigkeit(x As Double, y As Double) As Double
      x = x - PositionX ' X-Koordinate in Bezug auf den Transmitter umrechen

      Dim alpha = Math.Atan2(y, x)

      If alpha > PI2 Then ' > 90°, Näherung nur für Quadrant I
         alpha = Math.PI - alpha ' für Quadrant II spiegeln
      End If

      alpha = PI2 - alpha ' Bezug auf die Transmitter-Richtung, 0°-Transmitter zeigt nach oben, 0°-Koordiantensystem zeigt nach rechts

      ' http://www.sengpielaudio.com/Rechner-schallpegel.htm
      ' Berechnete Trendlinie
      Dim spl = -1.7544 * alpha ^ 4 + 7.0738 * alpha ^ 3 - 8.1318 * alpha ^ 2 - 0.0356 * alpha + 3.5304
      spl = spl / 3.5304 ' Maximalwert für spl ist 3.5304;  Normierung auf 1

      Return spl
   End Function

   ''' <summary>
   ''' Zeichnet den Transmitter auf der Bitmap.
   ''' </summary>
   ''' <param name="pb">Bitmap, auf der gezeichnet werden soll.</param>
   Sub draw(pb As ScaledBitmap)
      pb.FillRectangle(Brushes.Blue, PositionX, 0, Width, Height, ScaledBitmap.ObjectAlignment.Center)
   End Sub

End Class
